@@ -1,4 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-table-header.h
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #ifndef _E_TABLE_COLUMN_H_
 #define _E_TABLE_COLUMN_H_
 
@@ -43,7 +66,8 @@ typedef struct {
 	GtkObjectClass parent_class;
 
 	void (*structure_change) (ETableHeader *eth);
-	void (*dimension_change) (ETableHeader *eth, int col);
+	void (*dimension_change) (ETableHeader *eth, int width);
+	void (*expansion_change) (ETableHeader *eth);
 	int (*request_width) (ETableHeader *eth, int col);
 } ETableHeaderClass;
 
@@ -66,6 +90,7 @@ ETableCol   **e_table_header_get_columns            (ETableHeader *eth);
 int           e_table_header_get_selected           (ETableHeader *eth);
 
 int           e_table_header_total_width            (ETableHeader *eth);
+int           e_table_header_min_width              (ETableHeader *eth);
 void          e_table_header_move                   (ETableHeader *eth,
 						     int           source_index,
 						     int           target_index);
