@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-table-specification.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -21,18 +21,15 @@
  * 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include "e-table-specification.h"
 
 #include <stdlib.h>
 #include <string.h>
-
 #include <gtk/gtksignal.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
 
@@ -51,17 +48,17 @@ etsp_destroy (GtkObject *object)
 			gtk_object_unref (GTK_OBJECT (etsp->columns[i]));
 		}
 		g_free (etsp->columns);
+		etsp->columns = NULL;
 	}
 
 	if (etsp->state)
 		gtk_object_unref (GTK_OBJECT (etsp->state));
+	etsp->state                = NULL;
+
 	g_free (etsp->click_to_add_message);
+	etsp->click_to_add_message = NULL;
 
 	g_free (etsp->domain);
-
-	etsp->columns              = NULL;
-	etsp->state                = NULL;
-	etsp->click_to_add_message = NULL;
 	etsp->domain		   = NULL;
 
 	GTK_OBJECT_CLASS (etsp_parent_class)->destroy (object);
@@ -251,6 +248,8 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 		if (!strcmp (children->name, "ETableColumn")) {
 			ETableColumnSpecification *col_spec = e_table_column_specification_new ();
 
+			gtk_object_ref (GTK_OBJECT (col_spec));
+			gtk_object_sink (GTK_OBJECT (col_spec));
 			e_table_column_specification_load_from_node (col_spec, children);
 			list = g_list_append (list, col_spec);
 		} else if (specification->state == NULL && !strcmp (children->name, "ETableState")) {
@@ -260,6 +259,11 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 		}
 	}
 
+	if (specification->state == NULL) {
+		/* Make the default state.  */
+		specification->state = e_table_state_vanilla (g_list_length (list));
+	}
+
 	specification->columns = g_new (ETableColumnSpecification *, g_list_length (list) + 1);
 	for (list2 = list, i = 0; list2; list2 = g_list_next (list2), i++) {
 		specification->columns[i] = list2->data;
@@ -275,29 +279,21 @@ e_table_specification_load_from_node (ETableSpecification *specification,
  *
  * This routine stores the @specification into @filename.  
  *
- * Returns: 0 on success or -1 on error.
+ * Returns: the number of bytes written or -1 on error.
  */
 int
 e_table_specification_save_to_file (ETableSpecification *specification,
 				    const char          *filename)
 {
 	xmlDoc *doc;
-	int ret;
-	
+
 	g_return_val_if_fail (specification != NULL, -1);
 	g_return_val_if_fail (filename != NULL, -1);
 	g_return_val_if_fail (E_IS_TABLE_SPECIFICATION (specification), -1);
 	
-	if ((doc = xmlNewDoc ("1.0")) == NULL)
-		return -1;
-	
+	doc = xmlNewDoc ("1.0");
 	xmlDocSetRootElement (doc, e_table_specification_save_to_node (specification, doc));
-	
-	ret = e_xml_save_file (filename, doc);
-	
-	xmlFreeDoc (doc);
-	
-	return ret;
+	return xmlSaveFile (filename, doc);
 }
 
 /**
